#ifndef MENULIB_MENU_H
#define MENULIB_MENU_H

#include "math/vec.h"
#include "object.h"

namespace pwn {

//main interface

void	menuDisplayRect(const vec2f &pos, const vec2f &size, unsigned int color, int textureID);
void	menuDisplayRectTexCoord(const vec2f &pos, const vec2f &size, const vec2f &tcmin, const vec2f &tcmax, unsigned int color, int textureID);
void 	menuDisplayRectBorder(const vec2f &pos, const vec2f &size, unsigned int color, float pixWidth);
void 	menuDisplayText(const vec2f &pos, const vec2f &size, const char *str, unsigned int color = 0xFFFFFFFF, const vec2f &fontSize = vec2f(1.f));
vec2f 	menuTextSize(const char *str, const vec2f &maxsize, const vec2f &fontSize = vec2f(1.f));

//maybe i should subclass texture?
//then i'd have to deal with virtual function tables in the texture class.  n.g.
//maybe I should move this to the console class?
//or maybe I should make console dependant on the menu system ...
class MenuFontTexture {
public:
	int textureID;
	
	class Width {
	public:
		Width() : start(0), end(1) {}
		float start;
		float end;
	};
	Width widths[192];
};

class IMenuSystem {
public:
	IMenuSystem() {}
	virtual ~IMenuSystem() {}	//another CYGWIN trick . . . making the dtor virtual
	
	//system - for external programs to call
	virtual void init() = 0;
	virtual void shutdown() = 0;
	virtual void display() = 0;
	virtual bool getInput() = 0;
	virtual vec2f getSize() const = 0;
	
	//old menu class
	virtual Object * getRoot() = 0;
	
	//testing
	virtual MenuFontTexture *getFont() const = 0;
	
	virtual void setFocus(Object * m) = 0;

	//this is a horrible hack.
	//i need to redesign events for bubble-in and bubble-out
	//then tie all my editor input through the menus ...
	virtual bool keyWasTrapped() = 0;
	
	virtual void setCapture(Object *) = 0;
	virtual Object * getCapture() = 0;
	virtual void clearCapture() = 0;
};
extern IMenuSystem &menuSys;

};

#endif	//MENULIB_MENU_H
